package repos

import (
	"context"
	"gralias/models"
	"time"

	"github.com/jmoiron/sqlx"
)

type SessionsRepo interface {
	SessionByToken(ctx context.Context, token string) (*models.Session, error)
	SessionCreate(ctx context.Context, session *models.Session) error
	SessionUpdate(ctx context.Context, session *models.Session) error
	SessionDelete(ctx context.Context, token string) error
}

func (p *RepoProvider) SessionByToken(ctx context.Context, token string) (*models.Session, error) {
	db := getDB(ctx, p.DB)
	session := &models.Session{}
	// The lifetime in the DB is in seconds, but in the model it is in minutes.
	err := sqlx.GetContext(ctx, db, session, `SELECT id, updated_at, lifetime / 60 as lifetime, cookie_token, username FROM sessions WHERE cookie_token = ?`, token)
	if err != nil {
		return nil, err
	}
	return session, nil
}

func (p *RepoProvider) SessionCreate(ctx context.Context, session *models.Session) error {
	db := getDB(ctx, p.DB)
	// The lifetime in the model is in minutes, but in the DB it is in seconds.
	_, err := db.ExecContext(ctx, `INSERT INTO sessions (updated_at, lifetime, cookie_token, username) VALUES (?, ?, ?, ?)`,
		time.Now(), session.Lifetime*60, session.CookieToken, session.Username)
	return err
}

func (p *RepoProvider) SessionUpdate(ctx context.Context, session *models.Session) error {
	db := getDB(ctx, p.DB)
	// The lifetime in the model is in minutes, but in the DB it is in seconds.
	_, err := db.ExecContext(ctx, `UPDATE sessions SET updated_at = ?, lifetime = ? WHERE cookie_token = ?`,
		time.Now(), session.Lifetime*60, session.CookieToken)
	return err
}

func (p *RepoProvider) SessionDelete(ctx context.Context, token string) error {
	db := getDB(ctx, p.DB)
	_, err := db.ExecContext(ctx, `DELETE FROM sessions WHERE cookie_token = ?`, token)
	return err
}
